/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmultidimensionalminimizer.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVMultidimensionalMinimizer>

double QVMultidimensionalMinimizer::static_function (const gsl_vector *v, void *multidimensionalMinimizerPtr)
	{
	QVMultidimensionalMinimizer *multidimensionalMinimizer =
		(QVMultidimensionalMinimizer *) multidimensionalMinimizerPtr;

	/// @todo Use GSL vector to QVVector constructor when available
	for (int i = 0; i < ((int)v->size); i++)
		multidimensionalMinimizer->actual[i] = gsl_vector_get(v, i);

	return multidimensionalMinimizer->function(multidimensionalMinimizer->actual);
	}

bool QVMultidimensionalMinimizer::iterate(const int maxIterations, const double maxError)
	{
	// 1. Get first point and step direction
	/// @todo Use QVVector to GSL vector convert operator when available.
	gsl_vector *x = gsl_vector_alloc (actual.size());
	for(int i=0; i < actual.size(); i++)
		gsl_vector_set (x, i, actual[i]);
	
	/// @todo Use QVVector to GSL vector convert operator when available.
	gsl_vector *step_direction = gsl_vector_alloc (step.size());
	for(int i=0; i < step.size(); i++)
		gsl_vector_set (step_direction, i, step[i]);

	// 2. Create GSL minimization structures.
	gsl_multimin_fminimizer *s = gsl_multimin_fminimizer_alloc (gsl_multimin_fminimizer_nmsimplex, vectorSize);
	gsl_multimin_function multimin_function = { &static_function, vectorSize, this };
	gsl_multimin_fminimizer_set (s, &multimin_function, x, step_direction);

	// 3. Proceed to iterate.
	iterations = 0;
	int status = GSL_CONTINUE;
	while (status == GSL_CONTINUE && iterations++ <= maxIterations)
		if ((status = gsl_multimin_fminimizer_iterate (s)))
			break;
		else
			status = gsl_multimin_test_size (gsl_multimin_fminimizer_size (s), maxError);

	minimumValue = gsl_multimin_fminimizer_minimum(s);

	// 4. Release GSL minimization structures.
	gsl_multimin_fminimizer_free (s);
	gsl_vector_free (x);
	gsl_vector_free (step_direction);

	// 5. Return optimization status.
	return (status == GSL_SUCCESS);
	}

#include <qvmath/qvtensorindexator.h>
const QList<QVVector> QVMultidimensionalMinimizer::gridIterate(const QVVector &firstGridPoint, const QVVector &secondGridPoint, const int divisions, const int maxIterations, const double maxError)
	{
	Q_ASSERT(vectorSize == secondGridPoint.size());
	Q_ASSERT(firstGridPoint.size() == secondGridPoint.size());

	//////////////////////
	const QVVector step = (secondGridPoint - firstGridPoint)/(divisions-1);
	QVTensorIterator iterator(QVector<int>(vectorSize ,divisions));
	QList<QVVector> startingVectors;
	do	{
		QVVector startingVector(vectorSize);
		for (int i = 0; i < vectorSize; i++)
			startingVector[i] = firstGridPoint[i] + step[i] * iterator.getIndex(i);
		if (!startingVectors.contains(startingVector))
			startingVectors.append(startingVector);
		} while(iterator.nextVector());

	//////////////////////

	QList<QVVector> solutions;
	QVVector minVector;
	double	minValue = std::numeric_limits<double>::max();
	foreach(QVVector startingVector, startingVectors)
		{
	//do	{
		//QVVector startingVector(vectorSize);
		//for (int i = 0; i < vectorSize; i++)
		//	startingVector[i] = firstGridPoint[i] + step[i] * iterator.getIndex(i);
		setStartingVector(startingVector);
		if (!iterate(maxIterations, maxError))
			continue;

		solutions.append(getMinimum());

		//std::cout << startingVector << "\tf("<< getMinimum() << ") = " << getMinimumValue() << std::endl;

		if (getMinimumValue() < minValue)
			{
			minValue = getMinimumValue();
			minVector = getMinimum(); 
			}

		}// while(iterator.nextVector());

	//std::cout << "Minimum: f(" << minVector << ") = " << minValue << std::endl;

	setMinimum(minVector);
	setMinimumValue(minValue);
	//actual = minVector;
	//minimumValue = minValue;

	//std::cout << "Minimum: f(" << getMinimum() << ") = " << getMinimumValue() << std::endl;
	return solutions;
	}
